/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer;

import de.jtem.moebiusViewer.shape.Rectangle;

public abstract class AbstractBounder extends MoebiusGraphics {
  static final private long     serialVersionUID = 246483960508554831L;
  boolean                       bounded = false;
  public Rectangle              bound = new Rectangle ();

  public Rectangle getBound () {
    if (bounded) {
      return new Rectangle (bound);
    } 
    else {
      return null;
    } 
  } 

  public void add (Rectangle aBound) {
    if (bounded) {
      bound.unify (aBound);
    } 
    else {
      bound.set (aBound);
    } 
    bounded = true;
  } 

  public boolean isBounded () {
    return bounded;
  } 

  public void setBounded (boolean aBool) {
    bounded = aBool;
  } 

  public void initBounding () {
    setBounded (false);
  } 

  public Rectangle getBound (MoebiusShape aShape) {
    Rectangle   tmpBound = bound;
    Rectangle   result = new Rectangle ();

    bound = result;

    clear();
    push (aShape);
    initBounding ();
    aShape.draw (this);
    popShape ();

    bound = tmpBound;
    return result;
  } 

}

