/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;

import de.jtem.mfc.field.Complex;
import de.jtem.mfc.geometry.ComplexProjective1;

public abstract class AbstractDrawer extends MoebiusGraphicsWithPrimitives {

    private static final long     serialVersionUID = 938786947638905884L;
    private   ComplexProjective1    tu = new ComplexProjective1();
    private   Complex               u  = new Complex();

    public AbstractPSWriter getPSWriter() {
	return null;
    }

    public void draw( Graphics G, MoebiusShape aShape ) {
	clear();
	setGraphics(G);
	push (aShape);
	aShape.draw (this);
	popShape ();
    }

    public void draw( Graphics G, MoebiusTool aTool ) {
	setGraphics (G);
	aTool.draw (this);
    }


    public void drawColor( Color color ) {
        getGraphics().setColor( color );
    }

    public void drawCircle(double x, double y, double radius ) {
    	if( radius == 0 ) return;
        getGraphics().drawOval( (int)(x-radius),
				size.height - (int)(y + radius),
				2*(int)radius, 2*(int)radius);
    }

    public void drawLine(double x1, double y1, double x2, double y2) {
        getGraphics().drawLine( (int)x1, size.height - (int)y1, (int)x2, size.height -(int)y2);

    }

    public void drawArc( double x, double y,
			 double radius, double startAngle, double arcAngle ) {

	getGraphics().drawArc
	    ( (int)(x-radius), size.height - (int)(y + radius),
	      2*(int)radius, 2*(int)radius, (int)startAngle, (int)arcAngle);

    }


    public void drawString(String s, double x, double y) {

      drawColor(attributes.color);

      FontMetrics fontMetrics = getGraphics().getFontMetrics(); //uhhh
      double xPos, yPos;

      switch (attributes.horizontalTextLayout) {

        case Attributes.CENTER:
          xPos = x - fontMetrics.stringWidth(s) / 2;
          break;

        case Attributes.RIGHT:
          xPos = x - fontMetrics.stringWidth(s);
          break;

        case Attributes.LEFT:

        default:
          xPos = x;
      }

      switch (attributes.verticalTextLayout) {

        case Attributes.CENTER:
          yPos = y + fontMetrics.getDescent() / 2
              - fontMetrics.getAscent() / 2;
          break;

        case Attributes.BOTTOM:
          yPos = y + fontMetrics.getDescent();
          break;

        case Attributes.BASE:
        default:
          yPos = y;
          break;

        case Attributes.TOP:
          yPos = y - fontMetrics.getAscent();
      }

      getGraphics().drawString( s, (int)xPos, size.height - (int)yPos );
    }

    public void fillColor( Color color ) {
        getGraphics().setColor( color );
    }

    public void fillCircle( double x, double y, double radius ) {
    	if( radius == 0 ) return;
        getGraphics().fillOval( (int)(x-radius),
				size.height - (int)(y + radius),
				2*(int)radius, 2*(int)radius);
    }

    public void fillPolygon( double[] xPoints, double[] yPoints, int nPoints ) {

	int n = yPoints.length;

	int [] newXPoints = new int[n];
        int [] newYPoints = new int[n];

	for (int i=0; i<n; i++) {
	  newXPoints[i] = (int)(xPoints[i] + 0.5);
	  newYPoints[i] = size.height - (int)(yPoints[i]+0.5);
        }


        getGraphics().fillPolygon( newXPoints, newYPoints, nPoints );

    }

}

