/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.Vector;

import de.jtem.mfc.field.Complex;
import de.jtem.mfc.geometry.ComplexProjective1;
import de.jtem.moebiusViewer.shape.Rectangle;

public abstract class AbstractPSWriter extends MoebiusGraphicsWithPrimitives {
    private static final long     serialVersionUID = 1L;
    protected double[]            p = new double[4];
    protected double[]            tp = new double[4];
    MoebiusShape                  shape;
    float lineWidth = 1;
    protected PrintWriter         writer;
    String                        creator = "moebius viewer 2d";
    String                        title   = "";
    protected Rectangle           bound   = new Rectangle( 0, 0, 100, 100 );
    static Vector                 fontNames = new Vector ();
    static {
	fontNames.addElement ("SansSerif");
	fontNames.addElement ("Serif");
	fontNames.addElement ("Monospaced");
    } 
    private ComplexProjective1    tu = new ComplexProjective1();
    private Complex               u  = new Complex();    

    private float[]       colorComp = new float[3];
    static String         reference = 
	"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ 1234567890";
    static String         prolog = "%%BeginProlog\n" 
	+ "/BD {bind def} bind def\n" + "/D {def} BD\n" + "/ISOF {\n" 
	+ "	dup findfont dup length 1 add dict begin {\n" 
	+ "		1 index /FID eq {pop pop} {D} ifelse\n" 
	+ "	} forall /Encoding ISOLatin1Encoding D\n" 
	+ "	currentdict end definefont\n" + "} BD\n" + "/F [\n" 
	+ "	/Helvetica ISOF\n" + "	/Helvetica-Bold ISOF\n" 
	+ "	/Helvetica-Oblique ISOF\n" + "	/Helvetica-BoldOblique ISOF\n" 
	+ "	/Helvetica ISOF\n" + "	/Helvetica-Bold ISOF\n" 
	+ "	/Helvetica-Oblique ISOF\n" + "	/Helvetica-BoldOblique ISOF\n" 
	+ "	/Times-Roman ISOF\n" + "	/Times-Bold ISOF\n" 
	+ "	/Times-Italic ISOF\n" + "	/Times-BoldItalic ISOF\n" 
	+ "	/Courier ISOF\n" + "	/Courier-Bold ISOF\n" 
	+ "	/Courier-Oblique ISOF\n" + "	/Courier-BoldOblique ISOF\n" 
	+ "] D\n" + "/SF {\n" + "	F exch get exch scalefont setfont\n" + "	(" 
	+ reference + ") stringwidth pop div\n" 
	+ "	currentfont exch scalefont setfont} BD\n" + "%%EndProlog";

    public void setFont (Font f) {
	if (f.equals (attributes.font)) {
	    return;
	} 

	super.setFont (f);
	writeFont ();
    } 

    protected final void writeFont () {
	writer.println (getWidthOfReference (attributes.font) + " " 
			+ attributes.font.getSize () + " " 
			+ getFontIndex (attributes.font) + " SF");
    } 

    protected final double getWidthOfReference (Font aFont) {
	FontMetrics fontMetrics = getGraphics ().getFontMetrics (aFont);

	return fontMetrics.stringWidth (reference);
    } 

    protected final int getFontIndex (Font aFont) {
	return fontNames.indexOf (aFont.getName ()) * 4 + 4 + aFont.getStyle ();
    } 
       
    final void writeColor( Color color ) {
	writer.println ((((float)color.getRed ()  ) / 255) + " " +
			(((float)color.getGreen ()) / 255) + " " +
			(((float)color.getBlue () ) / 255) + " setrgbcolor");
    } 
     
    final void writeLineWidth( float lineWidth ) {
    	if(this.lineWidth == lineWidth) return;
    	this.lineWidth = lineWidth;
    	writer.println ( lineWidth + " setlinewidth");
    }
    
    public AbstractPSWriter( Attributes att ) {
	super();
	
	setAttributes (att);
    }
   
    public MoebiusShape getShape() {
	return shape;
    }

    public void setShape( MoebiusShape shape ) {
	this.shape = shape;
    }

    public Rectangle getBound() {
	return bound;
    } 
    
    public void setBound( Rectangle bound ) {
	this.bound = bound;
    }

    public String getCreator () {
	return creator;
    } 
    
    public void setCreator( String creator ) {
	this.creator = creator;
    }
    
    public String getTitle () {
	return title;
    }   

    public void setTitle( String title ) {
	this.title = title;
    }

    public void write( File aFile ) {
	
	try {
	    FileOutputStream  out = new FileOutputStream (aFile);

	    writer = new PrintWriter (out);

	    writer.println ("%!PS-Adobe-2.0 EPSF-1.2");
	    writer.println ("%%Pages: 1");
	    writer.println ("%%BoundingBox: " 
			    +  bound.x            + " " +  bound.y + " " 
			    + (bound.x + bound.w) + " " + (bound.y+bound.h) );
	    writer.println ("%%Creator: (" + creator + ")");
	    writer.println ("%%Title: " + title);
	    writer.println ("%%EndComments");
	    writer.println (prolog);
	    writer.println ("1 setlinecap");
	    writer.println ("1 setlinejoin");
	    writer.println ("1 setlinewidth");
	    writeFont ();
	    writeColor( attributes.color );
	    writer.println ("newpath");
	    writer.println ( bound.x            + " " +  bound.y            + " moveto");
	    writer.println ((bound.x + bound.w) + " " +  bound.y            + " lineto");
	    writer.println ((bound.x + bound.w) + " " + (bound.y + bound.h) + " lineto");
	    writer.println ( bound.x            + " " + (bound.y + bound.h) + " lineto");
	    writer.println ("closepath");
	    writer.println ("clip");

	    if( shape != null )
		shape.draw (this);
	    else
		drawString( "empty scene", bound.x + bound.w / 2, bound.y + bound.h / 2 );

	    //writer.println ("%%PSTrailer\n%end\ngrestore");
	    writer.println ("%%PSTrailer\n%end\n");
	    writer.close ();
	    out.close ();
	} 

	catch (java.io.IOException io) {
	    System.err.println (io);
	    
	    return;
	} 
    }
    
    protected final float x (double aX) {
	return (float) aX;
    } 
    
    protected final float y (double aY) {
	return (float) (aY); //(bound.h + 2 * bound.y - aY);
    } 

    public void drawColor( Color color ) {
	writeColor( color );
    }

    public void drawCircle(double x, double y, double radius ) {
    	if( radius == 0 ) return;
    	writeLineWidth( (float)this.attributes.lineWidth );
	writer.println ("newpath");
	writer.println( x(x) + " " + y(y) + " " + (float)radius + " 0 360 arc ");
	writer.println ("closepath");
	writer.println ("stroke");
    }

    public void drawLine(double x1, double y1, double x2, double y2) {
    writeLineWidth( (float)this.attributes.lineWidth );
	writer.println ("newpath");
	writer.println ( x(x1) + " " + y(y1) + " moveto");	
	writer.println ( x(x2) + " " + y(y2) + " lineto");
	writer.println ("stroke");
    }

    public void drawArc(double x, double y, double radius, 
			double startAngle, double arcAngle) {
	if( radius == 0 ) return;
	writeLineWidth( (float)this.attributes.lineWidth );
	writer.println ("newpath");
	writer.println( x(x) + " " + y(y) + " " + (float)radius + " " 
			+ Math.min( startAngle, startAngle+arcAngle ) + " " 
			+ Math.max( startAngle, startAngle+arcAngle ) + " arc ");
	writer.println ("stroke");
    }

    public void drawString( String str, double x, double y) {
	writer.println( x(x) + " " + y(y) + " moveto ");
	writer.println ("(" + str + ") show");
    }

    public void fillColor( Color color ) {
	writeColor( color );
    }

    public void fillCircle( double x, double y, double radius ) { 
    	if( radius == 0 ) return;
	writer.println ("newpath");
	writer.println( x(x) + " " + y(y) + " " + (float)radius + " 0 360 arc ");
	writer.println ("closepath");
	writer.println ("fill");       
    }
    
    public void fillPolygon( double [] x, double [] y, int nPoints ) {
	
	writer.println ("newpath");
	writer.println ( x(x[0]) + " " + y(y[0]) + " moveto");	
	for( int i=1; i<nPoints; i++ )
	    writer.println ( x(x[i]) + " " + y(y[i]) + " lineto");
	writer.println ("closepath");
	writer.println ("fill");

    }

}

