/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer;

import java.awt.event.MouseEvent;

import de.jtem.moebiusViewer.util.MoebiusShapeStack;
import de.jtem.moebiusViewer.util.MoebiusTransformStack;

public abstract class AbstractPicker extends MoebiusGraphics {
  static final private long     serialVersionUID = 5441014781766548850L;
  protected int                 xPick;
  protected int                 yPick;
  protected boolean             isPickable = false; // -> pickable?
  protected boolean             isPicked   = false; // -> picked?

  MoebiusShapeStack             pickPath;
  MoebiusTransformStack         pickTransformPath;

    public int getXPick () {
	return xPick;
    } 

    public void setXPick (int v) {
	this.xPick = v;
    } 

    public int getYPick () {
	return yPick;
    } 

    public void setYPick (int v) {
	this.yPick = v;
    } 

    public void setPickPoint (int aX, int aY) {
	xPick = aX;
	yPick = aY;
    } 

    public boolean getIsPicked () {
	return isPicked;
    } 

    public boolean isPicked () {
	return isPicked;
    } 
    public void setIsPicked (boolean v) {
	if (v) {
	    pickPath          = new MoebiusShapeStack( shapeStack );
	    pickTransformPath = new MoebiusTransformStack( transformStack );
	} 
	else {
	    pickPath          = null;
	    pickTransformPath = null;
	} 
	
	isPicked = v;
    } 

    public MoebiusShapeStack getPickPath() {
	return pickPath;
    } 

    public MoebiusTransformStack getPickTransformPath() {
	return pickTransformPath;
    }
    
    public MoebiusShape push (MoebiusShape aShape) {
	isPickable = aShape.getTools() != null;
	
	return super.push( aShape );
    } 

    public MoebiusShape popShape () {
	MoebiusShape shape = super.popShape();
	if ( shapeStack == null || shapeStack.empty()){ isPickable = false;}
	else{
	    isPickable = ((MoebiusShape)shapeStack.peek()).getTools() != null;
	}
	return shape;
    } 

    public void clearPickPath () {
	pickPath = null;
    } 

    public void initPicking (int x, int y) {
	xPick = x;
	yPick = size.height-y;
	
	setIsPicked (false);

	isPickable = true;
    } 

    public boolean pick (MouseEvent e, MoebiusShape aShape) {
	clear();

	initPicking ((int) e.getX (), (int) e.getY ());

	push (aShape);
	aShape.draw(this);
	popShape ();

	return getIsPicked ();
    }

}

