/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.io.Serializable;

public class Attributes implements Serializable, Cloneable {
    private static final long     serialVersionUID = 1L;
    transient protected Graphics  graphics;
    public static final int       LEFT = 1;
    public static final int       RIGHT = 2;
    public static final int       CENTER = 3;
    public static final int       TOP = 4;
    public static final int       BOTTOM = 5;
    public static final int       BASE = 6;

    public Color            color = Color.black;
    public int              pointRadius = 3;
    public int              pointOutline = 1;
    public boolean          usePointColor = false;
    public Color            pointColor = Color.red;
    public boolean          filled = false;
    public Font             font = new Font ("SansSerif", Font.PLAIN, 10);
    public int              horizontalTextLayout = LEFT;
    public int              verticalTextLayout = BOTTOM;
    public double           lineWidth = 1;
    public boolean          antiAliasing = true;

    public boolean getJava2() {
	return false;
    }

    public Graphics getGraphics () {
	return graphics;
    } 

    public void setGraphics (Graphics g) {
	graphics = g;
	if (graphics == null) return; 
	graphics.setFont (font);
	graphics.setColor (color);
    } 

    public int getHorizontalTextLayout () {
	return horizontalTextLayout;
    } 

    public void setHorizontalTextLayout (int aNum) {
	horizontalTextLayout = aNum;
    } 

    public int getVerticalTextLayout () {
	return verticalTextLayout;
    } 

    public void setVerticalTextLayout (int aNum) {
	verticalTextLayout = aNum;
    } 

    public Font getFont () {
	return font;
    } 

    public void setFont (Font f) {
	font = f;
	if (graphics != null) graphics.setFont (font); 
    } 

    public Color getColor () {
	return color;
    } 

    public void setColor (Color v) {
	color = v;
	if (graphics != null) graphics.setColor (color);
    } 

    public int getPointOutline () {
	return pointOutline;
    } 

    public void setPointOutline (int v) {
	pointOutline = v;
    }

    public int getPointRadius () {
	return pointRadius;
    }

    public void setPointRadius (int v) {
	  pointRadius = v;
    } 

    public boolean getUsePointColor () {
	return usePointColor;
    } 

    public void setUsePointColor (boolean v) {
	this.usePointColor = v;
    }

    public Color getPointColor () {
	return pointColor;
    } 

    public void setPointColor (Color c) {
	pointColor = c;
    }
    
    public boolean getFilled () {
	return filled;
    } 

    public void setFilled (boolean v) {
      this.filled = v;
    } 

    public void setLineWidth(double width){
	if (width >= 0){
	    lineWidth = width;
	}
    }
    
    public double getLineWidth(){
	return lineWidth;
    }

    public void setAntiAliasing(boolean b){
	antiAliasing = b;
    }

    public boolean getAntiAliasing(){
	return antiAliasing;
    }

    public Object clone () {
	try {
	    return super.clone ();
	} catch (java.lang.CloneNotSupportedException e1) {
	    return null;
	} 
    } 
}

