/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer;

import java.awt.BasicStroke;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.io.IOException;

public class Attributes2D extends Attributes {
    private static final long        serialVersionUID = 1L;
    transient protected  Graphics2D  graphics2d;
    transient public     BasicStroke stroke = new BasicStroke();

    public boolean getJava2() {
	return true;
    }

    private void readObject(java.io.ObjectInputStream in)
	throws IOException, ClassNotFoundException{
	in.defaultReadObject();
	setLineWidth(lineWidth);
    }
 
    public void setLineWidth(double width) {
	super.setLineWidth(width);

	if (stroke == null){
	    stroke = new BasicStroke((float) lineWidth);  
	}
	else if(stroke.getLineWidth() != ((float) lineWidth)){
	    stroke = new BasicStroke((float) lineWidth, stroke.getEndCap(), 
				     stroke.getLineJoin(), 
				     stroke.getMiterLimit(), 
				     stroke.getDashArray(), 
				     stroke.getDashPhase());
	}
	if (graphics2d != null) graphics2d.setStroke(stroke);
    }

    public void setGraphics(Graphics g) {
	graphics = g;
	graphics2d = (Graphics2D) g;
	if (graphics != null) resetGraphics();
    }
    
    public void resetGraphics(){
	graphics2d.setStroke(stroke);
	if (antiAliasing) {
	    graphics2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
					RenderingHints.VALUE_ANTIALIAS_ON);
	}
	else {
	    graphics2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
					RenderingHints.VALUE_ANTIALIAS_OFF);
	}
    }
}





