/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer;

import java.awt.Image;

import de.jtem.mfc.geometry.ComplexProjective1;
import de.jtem.mfc.geometry.HermitianCircle;

public class GenericPicker extends AbstractPicker {
    static final private long     serialVersionUID = 1L;
    AbstractDrawer                drawer;
    Image                         image;
    int                           pixValue;
    SinglePixelGrabber            pixGrabber;
 
    public void setDrawer( AbstractDrawer v) {
	this.drawer = v;
    } 

    public void setImage( Image i ) {
        if (image == i)
            return;
	this.image = i;
        if (pixGrabber != null)
            pixGrabber.reset(image);
        if (drawer != null)
            drawer.setGraphics(image.getGraphics());
    } 

    public void initPicking (int x, int y) {

        super.initPicking(x, y);
        yPick = y;

        drawer.getGraphics().setClip(x, y, 1, 1);
        drawer.getGraphics().clearRect(x, y, 1, 1);
        if(pixGrabber != null){
            pixGrabber.reset(x, y);
        }else{
            pixGrabber = new SinglePixelGrabber(image , x, y);
        }
        try{
            pixValue = pixGrabber.grabPixel();
        }
        catch (InterruptedException e){}
    }

    protected boolean pickedPixel(){
        pixGrabber.reset();
        //pixGrabber.reset(image, xPick, yPick);

        try{
            return !(pixValue == pixGrabber.grabPixel());
        }
        catch(InterruptedException e){}
        return false;
    }


    public MoebiusTransform push (MoebiusTransform T) {
        drawer.push(T);
        return super.push(T);
    }
    public MoebiusTransform popTransform () {
        drawer.popTransform();
        return super.popTransform();
    }
    public void clear() {
        super.clear();
        drawer.clear();
    }




    /* Moebius geometric primitives */
    public void point (ComplexProjective1 z) {
        if( !isPickable || isPicked ) {
            return;
        } 

        drawer.point(z);

        if(pickedPixel()){
            setIsPicked(true);
        }
    }
    public void lineSeg (ComplexProjective1 z1, ComplexProjective1 z2){ 
        if( !isPickable || isPicked ) {
            return;
        }     
        drawer.lineSeg(z1, z2);

        if(pickedPixel()){
            setIsPicked(true);
        }
    }
    public void text  (ComplexProjective1 z, String s ) {
        if( !isPickable || isPicked ) {
            return;
        }     
        drawer.text(z, s);

        if(pickedPixel()){
            setIsPicked(true);
        }
    }
    public void circle (HermitianCircle c) {
        if( !isPickable || isPicked ) {
            return;
        }     
        drawer.circle(c);

        if(pickedPixel()){
            setIsPicked(true);
        }
    }
    public void arc (ComplexProjective1 z1, ComplexProjective1 z2, ComplexProjective1 z3){
        if( !isPickable || isPicked ) {
            return;
        }     
        drawer.arc(z1, z2, z3);

        if(pickedPixel()){
            setIsPicked(true);
        }
    }
    
}

