/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer;

import java.io.Serializable;

import de.jtem.mfc.field.Complex;
import de.jtem.mfc.group.Moebius;


public class MoebiusTransform 
    extends de.jtem.mfc.group.Moebius
    implements Serializable,Cloneable {
    private static final long     serialVersionUID = 1L;
    private Complex z = new Complex(); 
    private Complex r = new Complex(); 
    
    public MoebiusTransform () {
        super ();
    }
    
    public MoebiusTransform (Moebius T) {
        super (T);
    }
    
    public MoebiusTransform apply (MoebiusTransform T) {
        MoebiusTransform     result = new MoebiusTransform (this);
  
        result.preConcatenate (T);      
        //        result.concatenate (T);
        return result;
    } 
    
    public void concatenate (MoebiusTransform T) {    
     
        this.assignTimes(T, this);
    } 
    
    public void preConcatenate (MoebiusTransform T) {
        
        this.assignTimes(this, T);
    } 
    
    

    /**
     * @deprecated 
     */
    public void transform (double[] srcPts, int srcOff, double[] dstPts, int dstOff, int numPts){ 
    
        for (int i = srcOff; i < 2* numPts; i +=2){
            z.assign(srcPts[i], srcPts[i + 1]);
            applyTo(z, r);
            dstPts[i] = r.getRe();
            dstPts[i+1] = r.getIm();
        } 
    } 

    /**
     * @deprecated
     */

    public void inverseTransform (double[] srcPts, int srcOff, double[] dstPts, int dstOff, int numPts){  
        for (int i = srcOff; i < 2* numPts; i +=2){
            z.assign(srcPts[i], srcPts[i + 1]);
            applyInverseTo(z, r);
            dstPts[i] = r.getRe();
            dstPts[i+1] = r.getIm();
        } 
         
    }
    

    public void translate (double tx, double ty) {

        MoebiusTransform m = new MoebiusTransform();
        m.assign(1, 0, tx, ty, 0, 0, 1, 0);
        concatenate(m);       
    } 
    
    public void rotate (double theta) {
        
        MoebiusTransform m = new MoebiusTransform();
        m.assign(Math.cos(theta), Math.sin(theta), 0, 0, 0, 0, 1, 0);
        concatenate(m);       
    } 
    
    public void scale (double s) {
    
        MoebiusTransform m = new MoebiusTransform();
        m.assign(Math.sqrt(s), 0, 0, 0, 0, 0, 1/Math.sqrt(s), 0);
        concatenate(m);       
    } 

  
  
  

}

