/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.graph;

import java.awt.Color;
import java.awt.Dimension;
import java.util.Vector;

import de.jtem.moebiusViewer.Attributes;
import de.jtem.moebiusViewer.MoebiusGraphics;
import de.jtem.moebiusViewer.shape.AbstractShape;

public class Graph extends AbstractShape {

    static private final long serialVersionUID = 1L;

    Vector value = new Vector();
    Vector color = new Vector();

    double  [][] v;
    Color   []   c;

    double minY, maxY, minX, maxX;
 	    
    public Graph() {
	super();	

	setPickable( false );
    }
    
    Color [] colorList = new Color[] { 
	Color.black,
	Color.magenta,
	Color.red,
	Color.blue,
	Color.gray,
	Color.white, 
	Color.lightGray,
	Color.pink,
	Color.orange,
	Color.darkGray,
	Color.yellow,
	Color.green,
	Color.cyan };
		    
    Color getUnusedColor() {
	for( int i=0; i<colorList.length; i++ )
	    if( -1 == color.indexOf( colorList[i] ) )
		return colorList[i];

	return Color.black;
    }

    public void addGraph( double [] y ) {
	
	value.add( y.clone() );
	color.add( getUnusedColor() );
	
	update();

	firePropertyChange( "invalidate" );
    }

    public void removeGraph( int index ) {
	
	value.remove( index );
	color.remove( index );

	update();

	firePropertyChange( "invalidate" );
    }

    public void setGraph( int index, double [] y ) {

	value.setElementAt( y.clone(), index );

	update();

	firePropertyChange( "invalidate" );
    }

    public Vector getValues(){
	return value;
    }

    void updateArrays() {

	v = new double[  value.size() ][];
	for( int i=0; i<v.length; i++ )
	    v[i] = (double [])value.elementAt( i );

	c = new Color[ color.size() ];
	for( int i=0; i<c.length; i++ )
	    c[i] = (Color)color.elementAt( i );
    }

    public double getMinY() {
	return minY;
    }

    public double getMaxY() {
	return maxY;
    }

    void updateBounds() {

	minX = 0;
	maxX = 0;
	
	if( v.length == 0 ) {
	    minY = maxY = 0;
	    return;
	}

	minY = Double.POSITIVE_INFINITY;
	maxY = Double.NEGATIVE_INFINITY;
	
	for( int i=0; i<v.length; i++ ) {
	    if( maxX < v[i].length - 1  )
		maxX = v[i].length - 1;

	    for( int j=0; j<v[i].length; j++ ){
		if( minY > v[i][j] )
		    minY = v[i][j];
		if( maxY < v[i][j] )
		    maxY = v[i][j];
	    }
	    
	}

//  	System.out.println( "minX = " + minX );
//  	System.out.println( "maxX = " + maxX );
//  	System.out.println( "minY = " + minY );
//  	System.out.println( "maxY = " + maxY );
    }

    double scaleX, scaleY;

    int minScaleY;
    int maxScaleY;

    int minScaleX;
    int maxScaleX;

    double xDenom, yDenom;

    void updateDenoms() {

	if( v.length != 0 ) {

	    scaleY = Math.pow( 10, Math.max
			       ( Math.floor( Math.log( Math.abs( minY ) ) / Math.log( 10 ) -0.001 ),
				 Math.floor( Math.log( Math.abs( maxY ) ) / Math.log( 10 ) -0.001 ) ) );

	    scaleX = Math.pow( 10, Math.max
			       ( Math.floor( Math.log( Math.abs( minX ) ) / Math.log( 10 ) -0.001 ),
				 Math.floor( Math.log( Math.abs( maxX ) ) / Math.log( 10 ) -0.001 ) ) );
	} else {

	    scaleY = 1;
	    scaleX = 1;
	}

	maxScaleY = (int)( ( Math.floor( maxY / scaleY ) + 1 ) );
	minScaleY = (int)( ( Math.floor( minY / scaleY )         ) );

	maxScaleX = (int)( ( Math.floor( maxX / scaleX ) + 1 ) );
	minScaleX = (int)( ( Math.floor( minX / scaleX )         ) );

	yDenom = ( maxScaleY - minScaleY ) * scaleY;
	xDenom = ( maxScaleX - minScaleX ) * scaleX;

	//System.out.println( scaleY + "  " + maxScaleY + "  " + minScaleY );
    }

    void update() {
	
	updateArrays();
	updateBounds();
	updateDenoms();
    }

    public void drawGrid( MoebiusGraphics g ) {

	Dimension size = g.getSize();

	double xFactor = size.width  / xDenom;
	double yFactor = size.height / yDenom;

	g.setColor( Color.gray );
	g.setLineWidth( 0.5 );

	g.setVerticalTextLayout(   Attributes.TOP );
	g.setHorizontalTextLayout( Attributes.RIGHT  );

	for( int i=minScaleY+1; i<=maxScaleY; i++ ) {

	    g.text( maxScaleX * scaleX * xFactor, i * scaleY * yFactor, (i * scaleY) + " " );
	}
 
	g.setHorizontalTextLayout( Attributes.CENTER  );

	for( int i=minScaleX; i<=maxScaleX; i++ ) {

	    g.text(  i * scaleX * xFactor, minScaleY * scaleY * yFactor, 
		     Integer.toString( (int)(i * scaleX))  );
	}
 

	for( int i=minScaleY; i<=maxScaleY; i++ ) {

	    g.lineSeg( minScaleX  * scaleX * xFactor, i * scaleY * yFactor,
		       maxScaleX  * scaleX * xFactor, i * scaleY * yFactor );

	}
 
	for( int i=minScaleX; i<=maxScaleX; i++ ) {

	    g.lineSeg( i * scaleX * xFactor, minScaleY * scaleY * yFactor,
		       i * scaleX * xFactor, maxScaleY * scaleY * yFactor );
	}
 
    }

    public void draw( MoebiusGraphics g ) {

	update();

	drawGrid( g );

	Dimension size = g.getSize();

	double xFactor = size.width  / xDenom;
	double yFactor = size.height / yDenom;
	
	for( int i=0; i<v.length; i++ ) {

	    g.setColor( c[i] );

	    for( int j=1; j<v[i].length; j++ ) {		
		g.lineSeg(  (j-1) * xFactor, v[i][j-1] * yFactor, 
			     j    * xFactor, v[i][j  ] * yFactor );			    
	    }
	    
	}
    }
    
    
    
}


