/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.moebius;

import de.jtem.mfc.field.Complex;
import de.jtem.mfc.geometry.ComplexProjective1;
import de.jtem.mfc.geometry.HermitianCircle;
import de.jtem.moebiusViewer.AbstractBounder;
import de.jtem.moebiusViewer.shape.Rectangle;

public class MoebiusBounder extends AbstractBounder {
  private static final long     serialVersionUID = 1L;
  private Rectangle             tmpRect = new Rectangle ();
  private HermitianCircle       tc = new HermitianCircle();
  private ComplexProjective1    tu = new ComplexProjective1();
  private ComplexProjective1    tw = new ComplexProjective1();
  private ComplexProjective1    tz = new ComplexProjective1();
  private Complex               z  = new Complex();
  private Complex               w  = new Complex();

    public void point (ComplexProjective1 aZ){

        tz.assignTimes(transform, aZ);
        tz.getB(z);

        if (!z.equals(0.,0.)){
           tz.projectTo(z);
            tmpRect.set (z.re, z.im, 0, 0);

            add (tmpRect);
        }
    }

    public void text( ComplexProjective1 aZ, String s ) {
	point( aZ );
    }

    public void circle( HermitianCircle aC ) {
        tc.assign(aC);
        tc.apply(transform);

        tc.euclideanCenter(z);
        double radius = Math.abs(tc.euclideanRadius());


        tmpRect.set (z.re - radius, z.im - radius, 2 * radius,
                 2 * radius);
        add (tmpRect);
    }


  public void lineSeg( ComplexProjective1 aZ, ComplexProjective1 aW ) {

      tz.assign(aZ);
      tz.assignTimes(transform, tz);
      tz.projectTo(z);

      tw.assign(aW);
      tw.assignTimes(transform, tw);
      tw.projectTo(w);
      if(Complex.equals(tz.bRe, tz.bIm, 0., 0.)){
          point(aW);
          return;
      }

      if(Complex.equals(tw.bRe, tw.bIm, 0., 0.)){
          point(aZ);
          return;
      }

      tmpRect.set (z.re < w.re ? z.re : w.re, z.im < w.im ? z.im : w.im,
                      Math.abs (z.re - w.re), Math.abs (z.im - w.im));

      add (tmpRect);
  }

    public void arc (ComplexProjective1 z1, ComplexProjective1 z2, ComplexProjective1 z3){
        tc.assignThrough(z1, z2, z3);
        tc.euclideanCenter(z);
        double radius = Math.abs(tc.euclideanRadius());
        if (z.isInfinite() || radius > 100000){
            lineSeg(z1, z3);
        }
        else{
            circle(tc);
        }
    }

}

