/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.moebius;

import de.jtem.mfc.field.Complex;
import de.jtem.mfc.geometry.ComplexProjective1;
import de.jtem.mfc.geometry.HermitianCircle;
import de.jtem.moebiusViewer.AbstractPicker;

public class MoebiusPicker extends AbstractPicker {

    private static final long     serialVersionUID = 1L;

    private HermitianCircle       tc = new HermitianCircle();

    private ComplexProjective1    tw = new ComplexProjective1();
    private ComplexProjective1    tz = new ComplexProjective1();

    private Complex               w = new Complex();
    private Complex               z = new Complex();
    
    public void lineSeg (ComplexProjective1 aZ, ComplexProjective1 aW) {

        if( !isPickable || isPicked ) {
            return;
        } 
        /*
         * TODO: Once is a line-width attribute, it should determine 
         *       halfPickWidth.
         */
        final double halfPickWidth = 2;
        final double x1t;
        final double x2t;
        final double y1t;
        final double y2t;
        final double xDiff;
        final double yDiff;
        final double norm;
        final double xNormal;
        final double yNormal;

        tz.assign(aZ);
        tz.assignTimes(transform, tz);
        tz.projectTo(z);
        
        tw.assign(aW);
        tw.assignTimes(transform, tw);
        tw.projectTo(w);


        xDiff = w.re - z.re;
        yDiff = w.im - z.im;
        norm = Math.sqrt(xDiff * xDiff + yDiff * yDiff);
        yNormal = xDiff / norm;
        xNormal = -yDiff / norm;
        
        if (isLeftOfLine(xPick, yPick, z.re, z.im, xDiff, yDiff)
            && isLeftOfLine(xPick, yPick, w.re, w.im, -xDiff, -yDiff)
            && isLeftOfLine(xPick, yPick, 
                            z.re - halfPickWidth * xNormal, 
                            z.im - halfPickWidth * yNormal,
                            xNormal, yNormal)
            && isLeftOfLine(xPick, yPick, 
                            z.re + halfPickWidth * xNormal, 
                            z.im + halfPickWidth * yNormal,
                            -xNormal, -yNormal)) {
            setIsPicked(true);
        }
    } 
    
    private final boolean isLeftOfLine(double xPoint,  double yPoint,
                                       double x0Line,  double y0Line, 
                                       double xNormal, double yNormal) {
        final double d = ((xPoint - x0Line) * xNormal 
                          + (yPoint - y0Line) * yNormal);
        return d > 0.0;
    }

    public void point (ComplexProjective1 aZ){
        if( !isPickable || isPicked ) {
            return;
        }     

        tz.assignTimes(transform, aZ);

        tz.getB(z);
      
        if (!z.equals(0.,0.)){
            tz.projectTo(z);

            int dist = (int) Math.sqrt ((xPick - z.re) * (xPick - z.re) 
                                         + (yPick -z.im) * (yPick -z.im));
           
            if (dist <= attributes.pointRadius) {
                setIsPicked (true);
            }
        }
    }

    public void text (ComplexProjective1 aZ, String s ) {
    }
    
    public void circle (HermitianCircle aC){
        if( !isPickable || isPicked ) {
            return;
        } 
 
        tc.assign(aC);
        tc.apply(transform);
        tc.euclideanCenter(z);
        int radius = (int)Math.abs(tc.euclideanRadius());

        int dist = (int) Math.sqrt ((xPick - z.re) * (xPick - z.re) 
                                    + (yPick - z.im) * (yPick - z.im));

        if (!attributes.filled && dist <= radius + 1 && dist >= radius - 1 
            || attributes.filled && dist <= radius){
            setIsPicked (true);
        } 

    }

    public void arc (ComplexProjective1 z1, ComplexProjective1 z2, ComplexProjective1 z3) {
    }

}

