/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.shape;

import de.jtem.mfc.geometry.ComplexProjective1;
import de.jtem.moebiusViewer.MoebiusGraphics;

public class Arc extends AbstractShape {
    private static final long     serialVersionUID = 1L;

    ComplexProjective1 z1 = new ComplexProjective1( 1, 0, 1, 0);
    ComplexProjective1 z2 = new ComplexProjective1( 0, 1, 1, 0);
    ComplexProjective1 z3 = new ComplexProjective1(-1, 0, 1, 0);
    protected boolean drawPoints;

    public Arc(){

        super();	
    }

    public void setZ1(ComplexProjective1 aZ){
        if (z1.equals(aZ))
            return;
        z1.assign(aZ);
        firePropertyChange("z1");
    }

    public ComplexProjective1 getZ1(){
        return new ComplexProjective1(z1);
    }

    public void setZ2(ComplexProjective1 aZ){
        if (z2.equals(aZ))
            return;
        z2.assign(aZ);
        firePropertyChange("z2");
    }

    public ComplexProjective1 getZ2(){
        return new ComplexProjective1(z2);
    }

    public void setZ3(ComplexProjective1 aZ){
        if (z3.equals(aZ))
            return;
        z3.assign(aZ);
        firePropertyChange("z3");
    }

    ComplexProjective1 getZ3(){
        return new ComplexProjective1(z3);
    }

    public boolean isDrawPoints() {return drawPoints;}

    public void setDrawPoints(boolean  v) {
        if (drawPoints == v)
            return;
        drawPoints = v;
        firePropertyChange("drawPoints");
    }
    
    public void draw(MoebiusGraphics context) {
        context.arc( z1, z2, z3 );
        if(drawPoints){
            context.point(z1);
            context.point(z2);
            context.point(z3);
        }
    }
    
}
