/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.shape;

import de.jtem.mfc.field.Complex;
import de.jtem.mfc.geometry.ComplexProjective1;
import de.jtem.mfc.geometry.HermitianCircle;
import de.jtem.moebiusViewer.MoebiusGraphics;
import de.jtem.moebiusViewer.tool.CircleDragTool;
import de.jtem.moebiusViewer.tool.RadiusTool;

public class Circle extends AbstractShape {
    private static final long     serialVersionUID = 1L;
    private final        Complex  euclideanCenter  = new Complex();

    HermitianCircle circle = new HermitianCircle();

    public Circle() {
	super();
        toolList.addElement( new RadiusTool() );
        toolList.addElement( new CircleDragTool() );
    }

    public Circle( double mX, double mY, double r ) {
        this();
	circle.assignEuclidean( mX, mY, r );
    }

    public Circle( HermitianCircle c ) {
	this();
        circle.assign( c );
    }

    public Circle( ComplexProjective1 z1, ComplexProjective1 z2, ComplexProjective1 z3){
        this();
        assign(z1, z2, z3);
    }


    /** @deprecated Use assign instead.*/
    public void set( double mX, double mY, double r) {
        assign(mX, mY, r);
    }


    public void assign( double mX, double mY, double r) {
	circle.euclideanCenter( euclideanCenter );
        if(euclideanCenter.equals(mX, mY) &&
           equals(getRadius(),r))
            return;

	circle.assignEuclidean( mX, mY, r );
        firePropertyChange( "circle data" );
    }



    public void assign(ComplexProjective1 z1, ComplexProjective1 z2, ComplexProjective1 z3){
        // TODO Wann PropertyChange verschicken?
        this.circle.assignThrough(z1, z2, z3);
        firePropertyChange( "circle data");
    }

    public HermitianCircle getHermitianCircle() {
        final HermitianCircle c = new HermitianCircle(circle);
	return c;
    }

    public void getHermitianCircle(HermitianCircle c){
        c.assign(circle);
    }

    public void setHermitianCircle( HermitianCircle circle ) {

        // TODO Wann PropertyChange verschicken?
	this.circle.assign( circle );
	firePropertyChange( "Hermitian Circle" );
    }

    public Complex getCenter() {
	Complex center = new Complex();
        circle.euclideanCenter( center );
	return center;
    }

    public void getCenter(Complex center){
        circle.euclideanCenter(center);
    }

    public void setCenter( Complex center ) {

        circle.euclideanCenter( euclideanCenter );
        if (center.equals(euclideanCenter))
            return;

	circle.assignEuclidean( center.re, center.im, circle.euclideanRadius() );
	firePropertyChange( "center" );
    }

    public double getCenterX() {
	circle.euclideanCenter( euclideanCenter );
	return euclideanCenter.re;
    }

    public void setCenterX( double centerX ) {

	circle.euclideanCenter( euclideanCenter );

        if(equals(euclideanCenter.re, centerX))
            return;
	circle.assignEuclidean( centerX, euclideanCenter.im, circle.euclideanRadius() );
        firePropertyChange( "centerX" );
    }

    public double getCenterY() {
        circle.euclideanCenter( euclideanCenter );
	return euclideanCenter.im;
    }

    public void setCenterY( double centerY ) {
	circle.euclideanCenter( euclideanCenter );
        if(equals(euclideanCenter.im, centerY) )
            return;

	circle.assignEuclidean( euclideanCenter.re, centerY, circle.euclideanRadius() );
	firePropertyChange( "centerY" );
    }

    public double getRadius() {
	return  circle.euclideanRadius();
    }

    public void setRadius( double radius ) {

        if (equals(radius, getRadius()))
            return;

	circle.euclideanCenter( euclideanCenter );
	circle.assignEuclidean( euclideanCenter.re, euclideanCenter.im, radius );
	firePropertyChange( "radius" );
    }

    public void draw(MoebiusGraphics context) {
	super.draw(context);
	context.circle( circle );
    }

}

