/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.shape;

import de.jtem.mfc.field.Complex;
import de.jtem.mfc.geometry.ComplexProjective1;
import de.jtem.moebiusViewer.Attributes;
import de.jtem.moebiusViewer.MoebiusGraphics;
import de.jtem.moebiusViewer.util.ColorGradient;

public class ColorGradientBox extends AbstractShape {
  private static final long     serialVersionUID = 1L;
  double                        x, y, w, h;

  ColorGradient                    colorMap;

  public ColorGradient getColorMap () {
    return colorMap;
  }

  public void setColorMap (ColorGradient v) {
    this.colorMap = v;
  }

  public double getH () {
    return h;
  }

  public void setH (double v) {
    this.h = v;
  }

  public double getW () {
    return w;
  }

  public void setW (double v) {
    this.w = v;
  }

  public double getX () {
    return x;
  }

  public void setX (double v) {
    this.x = v;
  }

  public double getY () {
    return y;
  }

  public void setY (double v) {
    this.y = v;
  }

  public ColorGradientBox (double aX, double aY, double aW, double aH,
                      ColorGradient aColorMap) {
    x = aX;
    y = aY;
    w = aW;
    h = aH;
    colorMap = aColorMap;
  }

  private ComplexProjective1    tz = new ComplexProjective1();
  private ComplexProjective1    tu = new ComplexProjective1();
  private Complex               z = new Complex();
  private Complex               u = new Complex();

  public void draw (MoebiusGraphics g) {
    super.draw (g);

    tz.assign(x, y, 1.0, 0.0);
    tz.assignTimes(g.getTransform (), tz);
    tz.projectTo(z);

    tu.assign(x+w, y-h, 1.0, 0.0);
    tu.assignTimes(g.getTransform (), tu);
    tu.projectTo(u);



    double W = 2 * Math.max (Math.abs (z.re - u.re),
                   Math.abs (z.im - u.im));

    int floorOfW = (int)Math.floor( W );

    for (int i = 0;i <= floorOfW;i++) {
      double    t = i / W;

      g.setColor (colorMap.getColor (t));
      g.lineSeg (x + t * w, y, x + t * w, y - h);
    }


    if( showLabel ) {

        g.setVerticalTextLayout(   Attributes.CENTER );
        g.setHorizontalTextLayout( Attributes.CENTER );

        g.setColor( java.awt.Color.black );

        g.text( x + w / 2., y - h / 2., label );
    }
  }

  public String toString () {
    return "(" + x + "," + y + "," + w + "," + h + ")";
  }

}

