/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.shape;

import de.jtem.function.complex.ComplexFunction;
import de.jtem.mfc.field.Complex;

/** This class implements the interfaces ComplexCycle and
 * ComplexPeriodicSequence in such a way that the closed plane polygons modeled
 * by these interfaces can be rendered and manipulated in 2D viewers from the
 * package de.jtem.moebiusViewer. */

public class ComplexFunctionOnIndexShape extends Polygon
    implements ComplexFunction.OnIndex {

    private static final long serialVersionUID = 1L;

    /** Creates an empty closed polygon with no points */

    public ComplexFunctionOnIndexShape() {
        setClosed(false);
    }

    public int getNumIndices() {
        return getNumOfPoints();
    }

    public Complex valueAt(int j) {
        return getCoords(j);
    }

    /** Become a copy of <code>c</code>. The domain is copied by reference,
     * the complex values are copied by value. */

    public void assign(ComplexFunction.OnIndex c) {
        int n = c.getNumIndices();
        setNumOfPoints(n);
        for (int j=0; j<n; j++) {
            setCoords( j, new Complex( c.valueAt(j) ));
        }
    }
}
