/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.shape;

import de.jtem.mfc.geometry.ComplexProjective1;
import de.jtem.moebiusViewer.MoebiusGraphics;

public class CoordinateSystem extends AbstractShape {

    private static final long     serialVersionUID = 1L;
    private double                xMin;
    private double                xMax;
    private double                yMin;
    private double                yMax;
    private double                originX = 0.0;
    private double                originY = 0.0;
    private double                xTickLength = 0.01;
    private double                xTickStep = 1.0;
    private double                yTickLength = 0.01;
    private double                yTickStep = 1.0;

    private boolean               drawUnitCircle = true;
    private boolean               drawTicks = true;

    private ComplexProjective1 start  = new ComplexProjective1();
    private ComplexProjective1 stop   = new ComplexProjective1();
    private ComplexProjective1 origin = new ComplexProjective1(0, 0, 1, 0);


    public CoordinateSystem(double theXMin, double theXMax,
                            double theYMin, double theYMax) {
        this.assign(theXMin, theXMax, theYMin, theYMax);
    }

    public CoordinateSystem() {
	this(-5,5,-5,5);
    }

    /**
     * Get the value of xMin.
     * @return Value of xMin.
     */
    public double getXMin() {
        return xMin;
    }

    /**
     * Set the value of xMin.
     * @param v  Value to assign to xMin.
     */
    public void setXMin(double  v) {

        if (equals(xMin, v))
            return;
        xMin = v;
        firePropertyChange( "xMin");
    }

    /**
     * Get the value of xMax.
     * @return Value of xMax.
     */
    public double getXMax() {
        return xMax;
    }

    /**
     * Set the value of xMax.
     * @param v  Value to assign to xMax.
     */
    public void setXMax(double  v) {
        if (equals(xMax, v))
            return;
        xMax = v;
        firePropertyChange( "xMax");
    }

    /**
     * Get the value of yMin.
     * @return Value of yMin.
     */
    public double getYMin() {
        return yMin;
    }

    /**
     * Set the value of yMin.
     * @param v  Value to assign to yMin.
     */
    public void setYMin(double  v) {
        if (equals(yMin, v))
            return;
        yMin = v;
        firePropertyChange( "yMin");

    }

    /**
     * Get the value of yMax.
     * @return Value of yMax.
     */
    public double getYMax() {return yMax;}

    /**
     * Set the value of yMax.
     * @param v  Value to assign to yMax.
     */
    public void setYMax(double  v) {
        if (equals(yMax, v))
            return;
        yMax = v;
        firePropertyChange( "yMax");

    }

    /**
     * Get the value of originX.
     * @return Value of originX.
     */
    public double getOriginX() {
        return originX;
    }

    /**
     * Set the value of originX.
     * @param v  Value to assign to originX.
     */
    public void setOriginX(double  v) {
        if (equals(originX, v))
            return;
        originX = v;
        firePropertyChange( "originX");

    }

    /**
     * Get the value of originY.
     * @return Value of originY.
     */
    public double getOriginY() {
        return originY;
    }

    /**
     * Set the value of originY.
     * @param v  Value to assign to originY.
     */
    public void setOriginY(double  v) {
        if (equals(originY, v))
            return;
        originY = v;
        firePropertyChange( "originY");

    }

    /**
     * Get the value of xTickLength.
     * @return Value of xTickLength.
     */
    public double getXTickLength() {
        return xTickLength;
    }

    /**
     * Set the value of xTickLength.
     * @param v  Value to assign to xTickLength.
     */
    public void setXTickLength(double  v) {
        if (equals(xTickLength, v))
            return;
        xTickLength = v;
        firePropertyChange( "xTickLength");
    }

    /**
     * Get the value of xTickStep.
     * @return Value of xTickStep.
     */
    public double getXTickStep() {
        return xTickStep;
    }

    /**
     * Set the value of xTickStep.
     * @param v  Value to assign to xTickStep.
     */
    public void setXTickStep(double  v) {
         if (equals(xTickStep, v))
            return;
        xTickStep = v;
        firePropertyChange( "xTickStep");
    }
    /**
     * Get the value of yTickLength.
     * @return Value of yTickLength.
     */
    public double getYTickLength() {
        return yTickLength;
    }

    /**
     * Set the value of yTickLength.
     * @param v  Value to assign to yTickLength.
     */
    public void setYTickLength(double  v) {
        if (equals(yTickLength, v))
            return;
        yTickLength = v;
        firePropertyChange( "yTickLength");
    }

    /**
     * Get the value of yTickStep.
     * @return Value of yTickStep.
     */
    public double getYTickStep() {
        return yTickStep;
    }

    /**
     * Set the value of yTickStep.
     * @param v  Value to assign to yTickStep.
     */
    public void setYTickStep(double  v) {
         if (equals(yTickStep, v))
            return;
        yTickStep = v;
        firePropertyChange( "yTickStep");
    }

    /**
     * Get the value of drawUnitCircle.
     * @return Value of drawUnitCircle.
     */
    public boolean getDrawUnitCircle() {
        return drawUnitCircle;
    }

    /**
     * Set the value of drawUnitCircle.
     * @param b  Value to assign to drawUnitCircle.
     */
    public void setDrawUnitCircle(boolean b) {
        if (drawUnitCircle == b)
            return;
        drawUnitCircle = b;
        firePropertyChange( "drawUnitCircle");
    }

    /**
     * Get the value of drawTicks.
     * @return Value of drawTicks.
     */
    public boolean getDrawTicks() {
        return drawTicks;
    }

    /**
     * Set the value of drawTicks.
     * @param v  Value to assign to drawTicks.
     */
    public void setDrawTicks(boolean b) {
        if (drawTicks == b)
            return;
        drawTicks = b;
        firePropertyChange( "drawTicks" );
    }

    public void assign (double theXMin, double theXMax, double theYMin, double theYMax) {

        if (equals(xMin, theXMin) && equals(xMax, theXMax) &&
            equals(yMin, theYMin) && equals(yMax, theYMax))
            return;
        xMin = theXMin;
        xMax = theXMax;
        yMin = theYMin;
        yMax = theYMax;
        firePropertyChange( "axes" );
    }

    public void draw (MoebiusGraphics g) {
        super.draw (g);

        if (lineWidth != 0) {
            origin.assign(originX, originY, 1, 0);
            start.assign(xMin + originX, originY, 1, 0);
            stop.assign(xMax + originX, originY, 1, 0);
            if ( (xMin == 0) || (xMax == 0))
                g.lineSeg(start, stop);
            else
                g.arc(start, origin, stop);
            start.assign(originX, yMin + originY, 1, 0);
            stop.assign(originX, yMax + originY, 1, 0);
            if ( (yMin == 0) || (yMax == 0))
                g.lineSeg(start, stop);
            else
                g.arc(start, origin, stop);
            if (drawUnitCircle)
                g.circle(originX, originY, 1);

            if (drawTicks) {
                if (xTickStep > 0) {
                    for (double tick = xMin; tick <= xMax; tick += xTickStep) {
                        g.lineSeg(tick + originX, originY - xTickLength,
                                  tick + originX, originY + xTickLength);
                    }
                }
                if (yTickStep > 0) {
                    for (double tick = yMin; tick <= yMax; tick += yTickStep) {
                        g.lineSeg(originX - yTickLength, tick + originY,
                                  originX + yTickLength, tick + originY);
                    }
                }
            }
        }
    }

  public String toString () {
    return "([" + xMin + ", " + xMax + "], [" + yMin + ", " + yMax + "])";
  }


} // CoordinateSystem
