/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.shape;

import de.jtem.moebiusViewer.Attributes;
import de.jtem.moebiusViewer.MoebiusGraphics;

public class LineSegment extends AbstractShape {
    
    private static final long     serialVersionUID = 1L;
    private double                x0;
    private double                x1;
    private double                y0;
    private double                y1;

    public LineSegment(double theX0, double theY0, 
                       double theX1, double theY1) {
        this.assign(theX0, theY0, theX1, theY1);
    }
    
    public LineSegment() {
	this(-1,0,1,0);
    }

    /**
     * Get the value of x0.
     * @return Value of x0.
     */
    public double getX0() {
        return x0;
    }
    
    /**
     * Set the value of x0.
     * @param v  Value to assign to x0.
     */
    public void setX0(double  v) {
        if (equals(x0, v))
            return;
        x0 = v;
        firePropertyChange("x0");
    }
    
    /**
     * Get the value of x1.
     * @return Value of x1.
     */
    public double getX1() {
        return x1;
    }
    
    /**
     * Set the value of x1.
     * @param v  Value to assign to x1.
     */
    public void setX1(double  v) {
        if (equals(x1, v))
            return;
        x1 = v;
        firePropertyChange("x1");
    }
       
    
    /**
     * Get the value of y0.
     * @return Value of y0.
     */
    public double getY0() {
        return y0;
    }
    
    /**
     * Set the value of y0.
     * @param v  Value to assign to y0.
     */
    public void setY0(double  v) {
        if (equals(y0, v))
            return;
        y0 = v;
        firePropertyChange("y0");
    }
    
    /**
     * Get the value of y1.
     * @return Value of y1.
     */
    public double getY1() {return y1;}
    
    /**
     * Set the value of y1.
     * @param v  Value to assign to y1.
     */
    public void setY1(double  v) {
        if (equals(y1, v))
            return;
        y1 = v;
        firePropertyChange("y1");
    }

    /** @deprecated */
    public void set (double theX0, double theY0, double theX1, double theY1) {
        assign(theX0, theY0, theX1, theY1);
    }

    public void assign (double theX0, double theY0, double theX1, double theY1) {

        if (equals(x0, theX0)&& equals(y0, theY0)&&
            equals(x1, theX1)&& equals(y1, theY1))
            return;

        x0 = theX0;
        y0 = theY0;
        x1 = theX1;
        y1 = theY1;
        firePropertyChange("endpoints");
    } 

    public void draw (MoebiusGraphics g) {
        super.draw (g);
        g.lineSeg(x0, y0, x1, y1);
        if( showLabel ) {
            g.setVerticalTextLayout(   Attributes.CENTER );
            g.setHorizontalTextLayout( Attributes.CENTER );
            // TODO: Text positioning will not work for non-eucl. linesegments.
            g.text(0.5 * (x0 + x1) , 0.5 * (y0 + y1), label );
        }
    } 

  public String toString () {
    return "((" + x0 + ", " + y0 + "), (" + x1 + ", " + y1 + "))";
  } 


} // LineSegment
