/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.shape;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import de.jtem.mfc.group.Moebius;
import de.jtem.moebiusViewer.MoebiusGraphics;
import de.jtem.moebiusViewer.MoebiusShape;
import de.jtem.moebiusViewer.MoebiusTransform;
import de.jtem.moebiusViewer.RenderTreeInfo;

public class MultipleMoebiusTransformer extends AbstractShape {

  private static final long     serialVersionUID = 1L;

  MoebiusShape shape;
  MoebiusTransform[] transform;

  public void propertyChange( PropertyChangeEvent e ) {

    firePropertyChange( e.getPropertyName(), e.getOldValue(), e.getNewValue() );
  }

  public MoebiusShape getShape () {
    return shape;
  } 

  public MoebiusTransform[] getTransform () {
    return transform;
  } 

  public MultipleMoebiusTransformer( Moebius [] transform ) {

    this.transform = new MoebiusTransform[ transform.length ];

    for( int i=0; i<transform.length; i++ ) 
      this.transform[i] = new MoebiusTransform( transform[i] );
  }


  public  MultipleMoebiusTransformer() { 

    this.transform = new MoebiusTransform[1];

    this.transform[0] = new MoebiusTransform();
  }

  public void setShape( MoebiusShape newShape ) {
    if( shape == newShape )
      return;

    if( RenderTreeInfo.contains( newShape, this ) )
      throw new IllegalArgumentException(" generated loop in render tree" );
    
    MoebiusShape oldShape = shape;

    shape = newShape;

    if( oldShape != null ) {

      try {
	oldShape.getClass().getMethod( "removePropertyChangeListener", new Class[] { PropertyChangeListener.class } )
	  .invoke( oldShape, new Object[] { this } );
      }
      catch( Exception e ) { }
      
      firePropertyChange( "removed shape", oldShape, null );       
    }

    if( newShape != null ) {

      try {
	newShape.getClass().getMethod( "addPropertyChangeListener", new Class[] {  PropertyChangeListener.class } )
	  .invoke( newShape, new Object[] { this } );
      }
      catch( Exception e ) { }
      
      firePropertyChange( "added shape",   newShape, null );       
    }


  }

  public void draw (MoebiusGraphics G) {
    if( shape == null )
      return;

    G.push (shape);      

    for( int i=0; i<transform.length; i++ ) {
      G.push (transform[i]);
      shape.draw (G);      
      G.popTransform ();
    }
    G.popShape ();
  }     
}

