/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.shape;

import de.jtem.mfc.field.Complex;
import de.jtem.moebiusViewer.MoebiusGraphics;

public class PointOnCircle extends Point {
    private static final long serialVersionUID = 1L;

    public Circle circle;
    public boolean drawCircle = true;

    public PointOnCircle() {
	this(1,0,0,0,1);
    }

    public PointOnCircle(double posX, double posY,
			 double mX, double mY, double r) {
	circle = new Circle(mX, mY, r);
	circle.setPickable(false);
	set(posX, posY);
    }

	/** @deprecated */
    public void set(double posX, double posY) {
	Complex center = circle.getCenter();
	double vecX = posX - center.re;
	double vecY = posY - center.im;
	double factor = circle.getRadius() / Math.sqrt(vecX * vecX + vecY * vecY);
	super.setCoords( center.re + factor * vecX,
		   center.im + factor * vecY );
    }

    public double getRadius() {
	return circle.getRadius();
    }

    public void setRadius( double r ) {
	circle.setRadius(r);
	set(getX(), getY());
	firePropertyChange( "radius" );
    }

    public void draw(MoebiusGraphics context) {
	if (drawCircle) circle.draw(context);
	super.draw(context);
    }

    public boolean getDrawCircle() {
	return drawCircle;
    }

    public void setDrawCircle(boolean b) {
	drawCircle = b;
    }

    public Circle getCircle() {
	return circle;
    }
}

