/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.shape;

import java.awt.Color;

import de.jtem.mfc.field.Complex;
import de.jtem.mfc.geometry.ComplexProjective1;
import de.jtem.moebiusViewer.MoebiusGraphics;
import de.jtem.moebiusViewer.tool.PointSetDragTool;

public class PointSet extends AbstractPointSet {
    private static final long     serialVersionUID = 1L;


    public PointSet (){
	toolList.add( 0, new PointSetDragTool() );
    }

    public PointSet(int n){
        this();
        setNumOfPoints(n);
    }

    public PointSet( double[] coords ) {
        this();
        assignCoords(coords);
    }

    public PointSet(Complex[] coords){
        this();
        assignCoords(coords);
    }

    public PointSet( ComplexProjective1 [] coords ) {
        this();
        assignCoords(coords);
    }

    public boolean isLabelPointsWithIndex(){
        return super.isLabelPointsWithIndex();
    }

    public void setLabelPointsWithIndex( boolean b){
        super.setLabelPointsWithIndex( b);
    }

    public int getNumOfPoints () {
	return super.getNumOfPoints();
    }

    public void setNumOfPoints (int newNumOfPoints) {
        super.setNumOfPoints(newNumOfPoints);
    }

    /** @deprecated */
    void setNumOfPointss( int newNumOfPoints, boolean assignDefault ) {

        super.setNumOfPoints(newNumOfPoints, assignDefault);
    }

    protected void setNumOfPoints( int newNumOfPoints, boolean assignDefault ) {
        numOfPoints = getNumOfPoints();

        if (newNumOfPoints == numOfPoints)
            return;
        final int min = newNumOfPoints < numOfPoints
            ? newNumOfPoints :numOfPoints;
        super.setNumOfPoints(newNumOfPoints, false);

        if (assignDefault){
            for (int i = min; i<numOfPoints; i++){
                aRe[i] = i;
                aIm[i] = 0;
                bRe[i] = 1;
                bIm[i] = 0;
            }
        }
    }
    /* methods for ComplexProjective coords */

    public void setProjectiveCoords (int index, double aRe1, double aIm1,
                           double bRe1, double bIm1){
        super.setProjectiveCoords(index, aRe1, aIm1, bRe1, bIm1);
    }

    public void setProjectiveCoords( int index, final ComplexProjective1 z ) {
        super.setProjectiveCoords(index, z);
    }

    public void getProjectiveCoords( ComplexProjective1 [] coords ) {
      super.getProjectiveCoords( coords );
    }

    public ComplexProjective1[] getProjectiveCoords() {
	return super.getProjectiveCoordsAsArray1d();
    }

    public void assignCoords( ComplexProjective1 [] coords ) {
        super.assignCoords(coords);
    }

    public void getProjectiveCoords(int index, ComplexProjective1 coords){
        super.getProjectiveCoords(index, coords);
    }

    public ComplexProjective1 getProjectiveCoords( int index ) {
        return super.getProjectiveCoords(index);
    }

     public Complex getCoords( int index ) {
        return super.getCoords(index);
    }

   /* methods for complex coords */

    public void setCoords( int index, final Complex coords ) {
        super.setCoords(index, coords);
    }

    public void getCoords(int index, Complex coords){
        super.getCoords(index, coords);
    }

    public void getCoords(Complex[] coords){
        super.getCoords(coords);
    }

    public Complex [] getCoords() {
	return super.getCoordsAsArray1d();
    }

    /** @deprecated */
    public void setCoords( Complex [] coords ) {
        assignCoords(coords);
    }

    public void assignCoords(Complex[] coords){
        super.assignCoords(coords);
    }



    /* methods for euclidean coords */

    /** @depricated  -- use bean conform parameter version instead */
    public void set (double aX, double aY, int anIndex) {
	setCoords(anIndex, aX, aY);
    }

    /** @deprecated */
    public void set (int anIndex, double aX, double aY) {
	setCoords( anIndex, aX, aY );
    }


    /** @deprecated */
    public void setXY (int anIndex, double aX, double aY) {
        setCoords(anIndex, aX, aY);
    }

    public void setCoords(int index, double x, double y ) {
        super.setCoords(index, x, y);
    }

    public double getX( int index ) {
        return super.getX(index);
    }

    public void setX( int index, double x ) {
        super.setX(index, x);
    }

    public double [] getX() {
        return super.getX();
    }

    public void setX( double [] x ) {
        super.setX(x);
    }

    public double getY( int index ) {
        return super.getY(index);
    }

    public void setY( int index, double y ) {
        super.setY(index, y);
    }

    public double [] getY() {
        return super.getY();
    }

    public void setY( double [] y ) {
        super.setY(y);
    }

    public double[] getXY () {
	return super.getXY();
    }

    /** @deprecated */
    public void setXY( double[] aXY ) {
        assignCoords(aXY);
    }

    public void assignCoords( double[] coords ) {
        super.assignCoords(coords);
    }

    /* methods for PointColor */
    public Color [] getPointColor() {
	return super.getPointColor();
    }

    public void setPointColor (Color[] aPointColor) {
        super.setPointColor(aPointColor);
    }

    public Color getPointColor (int anIndex) {
	return super.getPointColor(anIndex);
    }

    public void setPointColor (int anIndex, Color aPointColor) {
        super.setPointColor (anIndex, aPointColor);
    }

    public Color getDefaultPointColor() {
	return super.getDefaultPointColor();
    }

    public void setDefaultPointColor( Color color ) {
      super.setDefaultPointColor( color );
    }

    /* draw from interface shape */
    public void draw (MoebiusGraphics g) {
           super.draw(g);
           drawPoints(g);
    }

}

