/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.shape;

import de.jtem.moebiusViewer.MoebiusGraphics;

public class Polygon
    extends PointSet {
    private static final long serialVersionUID = 1L;

    protected boolean closed;

    public Polygon() {
        super();
    }

    public Polygon(int n) {
        super(n);
    }

    public boolean isClosed() {
        return closed;
    }

    public void setClosed(boolean v) {
        if (closed == v) {
            return;
        }
        closed = v;
        firePropertyChange("closed");
    }

    public void draw(MoebiusGraphics g) {
        super.draw(g);

    }

    protected void drawPoints(MoebiusGraphics g) {

        if (numOfPoints < 2) {
            super.drawPoints(g);
            return;
        }
        if (lineWidth != 0) {
            double aR1 = aRe[0];
            double aI1 = aIm[0];
            double bR1 = bRe[0];
            double bI1 = bIm[0];

            for (int i = 1; i < numOfPoints; i++) {
                double aR2 = aRe[i];
                double aI2 = aIm[i];
                double bR2 = bRe[i];
                double bI2 = bIm[i];
                g.lineSeg(aR1, aI1, bR1, bI1, aR2, aI2, bR2, bI2);
                aR1 = aR2;
                aI1 = aI2;
                bR1 = bR2;
                bI1 = bI2;
            }

            if (closed) {
                g.lineSeg(aR1, aI1, bR1, bI1, aRe[0], aIm[0], bRe[0], bIm[0]);
            }
        }
        super.drawPoints(g);
    }

    public Polygon reverse() {
    	Polygon reverse = new Polygon(numOfPoints); 
    	for( int i=0, j=numOfPoints-1; i<numOfPoints; i++, j-- ) {
    		reverse.aRe[j] = aRe[i];
    		reverse.aIm[j] = aIm[i];
    		reverse.bRe[j] = bRe[i];
    		reverse.bIm[j] = bIm[i];
    		reverse.pointColor[j] = pointColor[i];
    	}
    	return reverse;
    }
    
    public Polygon copy() {
    	Polygon copy = new Polygon( numOfPoints );
    	
    	System.arraycopy( aRe, 0, copy.aRe, 0, numOfPoints );
    	System.arraycopy( aIm, 0, copy.aIm, 0, numOfPoints );
    	System.arraycopy( bRe, 0, copy.bRe, 0, numOfPoints );
    	System.arraycopy( bIm, 0, copy.bIm, 0, numOfPoints );
    	System.arraycopy( pointColor, 0, copy.pointColor, 0, numOfPoints );
    	
    	return copy;
    }
    
    public Polygon concat( Polygon other ) {
    	Polygon sum = new Polygon( numOfPoints + other.numOfPoints );
    	
    	System.arraycopy( aRe, 0, sum.aRe, 0, numOfPoints );
    	System.arraycopy( aIm, 0, sum.aIm, 0, numOfPoints );
    	System.arraycopy( bRe, 0, sum.bRe, 0, numOfPoints );
    	System.arraycopy( bIm, 0, sum.bIm, 0, numOfPoints );
    	System.arraycopy( pointColor, 0, sum.pointColor, 0, numOfPoints );
    	
    	System.arraycopy( other.aRe, 0, sum.aRe, numOfPoints, other.numOfPoints );
    	System.arraycopy( other.aIm, 0, sum.aIm, numOfPoints, other.numOfPoints );
    	System.arraycopy( other.bRe, 0, sum.bRe, numOfPoints, other.numOfPoints );
    	System.arraycopy( other.bIm, 0, sum.bIm, numOfPoints, other.numOfPoints );
    	System.arraycopy( other.pointColor, 0, sum.pointColor, numOfPoints, other.numOfPoints );
    	
    	return sum;
    }
}
