/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.shape;

import java.io.Serializable;
import java.util.Vector;

import de.jtem.moebiusViewer.MoebiusGraphics;
import de.jtem.moebiusViewer.MoebiusShape;

public class Rectangle
    implements Serializable, Cloneable, MoebiusShape{

  private static final long     serialVersionUID = 1L;
  public double                 x, y, w, h;

  public Rectangle () {
      this(-1,-1,2,2);
  }

  public Rectangle (Rectangle aRectangle) {
    set (aRectangle);
  }

  public Rectangle (double aX, double aY, double aW, double aH) {
    set (aX, aY, aW, aH);
  }


  public void set (Rectangle aRectangle) {
    x = aRectangle.x;
    y = aRectangle.y;
    w = aRectangle.w;
    h = aRectangle.h;
  } 

  public void set (double aX, double aY, double aW, double aH) {
    x = aX;
    y = aY;
    w = aW;
    h = aH;
  } 

  public void unify (Rectangle aRectangle) {
    if (aRectangle == null) {
      return;
    } 

    double      X = x + w;
    double      Y = y + h;

    x = Math.min (x, aRectangle.x);
    y = Math.min (y, aRectangle.y);
    w = Math.max (X, aRectangle.x + aRectangle.w) - x;
    h = Math.max (Y, aRectangle.y + aRectangle.h) - y;
  } 

  public double[] getCorners () {
    double[]    corners = new double[8];

    corners[0] = x;
    corners[1] = y;
    corners[2] = x + w;
    corners[3] = y;
    corners[4] = x + w;
    corners[5] = y + h;
    corners[6] = x;
    corners[7] = y + h;

    return corners;
  } 

  public void setCorners (double[] corners) {
    x = Math.min (Math.min (Math.min (corners[0], corners[2]), corners[4]), 
                  corners[6]);
    y = Math.min (Math.min (Math.min (corners[1], corners[3]), corners[5]), 
                  corners[7]);
    w = 
      Math
      .max (Math.max (Math.max (corners[0], corners[2]), corners[4]), 
            corners[6]) - x;
    h = 
      Math
      .max (Math.max (Math.max (corners[1], corners[3]), corners[5]), 
            corners[7]) - y;
  } 
    

  public void draw (MoebiusGraphics g) {

    double[]    corner = getCorners ();

    g.lineSeg (corner[0], corner[1], corner[2], corner[3]);
    g.lineSeg (corner[2], corner[3], corner[4], corner[5]);
    g.lineSeg (corner[4], corner[5], corner[6], corner[7]);
    g.lineSeg (corner[6], corner[7], corner[0], corner[1]);
  } 


  public Vector getTools () {
    return null;
  }


  public String toString () {
    return super.toString () + " x=" + x + " y=" + y + " width=" + w 
           + " height=" + h;
  } 
}

