/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.shape;

import java.awt.Color;
import java.awt.Font;

import de.jtem.moebiusViewer.Attributes;
import de.jtem.moebiusViewer.MoebiusGraphics;

public class Text extends AbstractShape {
  private static final long     serialVersionUID = 1L;
  Font                          font = new Font ("Helvetica", Font.PLAIN, 10);
  Color                         fontColor = Color.black;
  int                           horizontalTextLayout = Attributes.LEFT;
  int                           verticalTextLayout   = Attributes.TOP;
  double                        x;
  double                        y;

  public Text () {
    showLabel = true;
  }

  public Text (String aLabel) {
    this ();

    label = aLabel;
  }

  public double getX () {
    return x;
  } 

  public void setX (double v) {

      if (equals(x, v))
          return;

      x = v;
      firePropertyChange("position");
  } 

  public double getY () {
    return y;
  } 

  public void setY (double v) {
      if (equals(y, v))
          return;

      y = v;
      firePropertyChange("position");
  } 

  public int getHorizontalTextLayout () {
    return horizontalTextLayout;
  } 

  public void setHorizontalTextLayout (int aNum) {
      if (horizontalTextLayout == aNum)
          return;
      horizontalTextLayout = aNum;
      firePropertyChange("layout");
  } 

  public int getVerticalTextLayout () {
    return verticalTextLayout;
  } 

  public void setVerticalTextLayout (int aNum) {
      if (verticalTextLayout == aNum)
          return;
      verticalTextLayout = aNum;
      firePropertyChange("layout");
  } 

  public Font getFont () {
    return font;
  } 

  public void setFont (Font f) {
      if (font.equals(f))
          return;
      font = f;
      firePropertyChange("font");
  } 

  public Color getFontColor () {
    return fontColor;
  } 

  public void setFontColor (Color c) {
      if (fontColor.equals(c))
          return;
      fontColor = c;
      firePropertyChange("fontColor");
  } 

  public void draw (MoebiusGraphics context) {
    super.draw (context);
    context.setFont (font);
    context.setHorizontalTextLayout (horizontalTextLayout);
    context.setVerticalTextLayout (verticalTextLayout);
    context.text (x, y, label);
  } 

}

