/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.shape;

import de.jtem.moebiusViewer.MoebiusGraphics;

public class Triangulation extends PointSet {
  private static final long     serialVersionUID = 1L;
  int[]                         index;

  /**
   * Get the value of index.
   * @return Value of index.
   */

  public int[] getIndex () {
    return index;
  } 

  /**
   * Set the value of index.
   * @param v  Value to assign to index.
   */

  public void setIndex (int[] v) {
    if (v.length % 3 != 0) {
      throw new IllegalArgumentException ("length of index array must be multiple of 3");
    } 

    numElements = v.length / 3;
    this.index = v;
  } 

  int   numElements;

  /**
   * Get the value of numElements.
   * @return Value of numElements.
   */

  public int getNumElements () {
    return numElements;
  } 

  /**
   * Set the value of numElements.
   * @param v  Value to assign to numElements.
   */

  public void setNumElements (int v) {
    this.numElements = v;
  } 

  public Triangulation () {
      setNumOfPoints(4);
    int[]       anIndex = new int[] {
      0, 1, 2, 3, 2, 1
    };

    setIndex (anIndex);
    setCoords (3, 0., 0.);
    setCoords (2, 1., 0.);
    setCoords (1, 0., 1.);
    setCoords (0, 1., 1.);
  }

  public void draw (MoebiusGraphics g) {
      super.draw (g);
  }

  protected void drawPoints(MoebiusGraphics g){

      g.setColor( color );
      
      for (int i = 0, k = 0;i < numElements;i++) {
          double    x0 = getX(index[k]);
          double    y0 = getY(index[k++]);
          double    x1 = getX(index[k]);
          double    y1 = getY(index[k++]);
          double    x2 = getX(index[k]);
          double    y2 = getY(index[k++]);
          
          g.lineSeg (x0, y0, x1, y1);
          g.lineSeg (x1, y1, x2, y2);
          g.lineSeg (x2, y2, x0, y0);
        } 
      super.drawPoints(g);
  } 

}

