/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.tool;

import java.awt.event.MouseEvent;

import de.jtem.moebiusViewer.MoebiusShape;
import de.jtem.moebiusViewer.MoebiusTransform;
import de.jtem.moebiusViewer.shape.MoebiusTransformer;
import de.jtem.moebiusViewer.util.MoebiusTransformStack;

public abstract class AbstractMoebiusTransformerTool extends AbstractTool {

    private static final long serialVersionUID = 1L;
    
    protected MoebiusTransformer context;

    protected MoebiusTransform     currentTransform;
    protected MoebiusTransform     firstTransform = new MoebiusTransform();
    
    public void setContext( MoebiusShape context ) {

	super.setContext( context );

	this.context = (MoebiusTransformer)super.context;
    }
    
    protected void updateContextTransform() {
	super.updateContextTransform();
	
	if( context != null && transformPath.peek() != context.getTransform() )
	    throw new RuntimeException( "lost transform of context; select background again" );
    }
    
    public void setTransformPath( MoebiusTransformStack transformPath ) {
	
	this.transformPath = new MoebiusTransformStack( transformPath );
		
	for( int i=0; i<this.transformPath.size(); i++ ) {
	    if( context.getTransform() == transformPath.elementAt(i) ) {
		this.transformPath.setSize( i+1 );
		break;
	    }
	}

	updateContextTransform();
    }


    public void mousePressed (MouseEvent e) {
	super.mousePressed (e);
	
	currentTransform = context.getTransform();

	firstTransform.assign( currentTransform );
    } 

} 

