/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.tool;


import java.awt.event.MouseEvent;

import de.jtem.mfc.field.Complex;
import de.jtem.moebiusViewer.AbstractPicker;
import de.jtem.moebiusViewer.shape.PointSet;
/**
 * @author schmies
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public abstract class AbstractPointSetTool extends AbstractTool {

	private static final long serialVersionUID = 1L;

	int indexOfPointToDrag = -1;

	AbstractPicker pickerContext;
	
	PointSet selection;

	protected AbstractPointSetTool( String label ) {
		setLabel( label );
	}

	public void mousePressed(MouseEvent e) {
		super.mousePressed( e );

		pickerContext = viewer.getPickerContext();

		pickerContext.setTransformStack( transformPath );

		pickerContext.initPicking ((int) e.getX (), (int) e.getY ());
		
		selection = (PointSet) (viewer.getSelection());


		int numOfPoints = selection.getNumOfPoints();

		pickerContext.setPointRadius( selection.getPointRadius() );

		for( int i=0; i<numOfPoints; i++ ) {

			pickerContext.point( selection.getX(i), selection.getY(i) );
			if( pickerContext.getIsPicked() ) {
				indexOfPointToDrag = i;
				return;
			}
		}

		indexOfPointToDrag = -1;
	}

	abstract protected void mouseDragAction( PointSet selection, int index, Complex newPosition );
	
	public void mouseDragged(MouseEvent e) {
		super.mouseDragged( e );

		if( indexOfPointToDrag != -1 ) {
			mouseDragAction( selection, indexOfPointToDrag, newPick );
		}
	}

	public void mouseReleased(MouseEvent e) {
		selection = null;
		indexOfPointToDrag = -1;
	}
}
