/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.tool;

import java.awt.Dimension;
import java.awt.event.MouseEvent;

import de.jtem.mfc.field.Complex;
import de.jtem.moebiusViewer.MoebiusGraphics;
import de.jtem.moebiusViewer.MoebiusShape;
import de.jtem.moebiusViewer.MoebiusTool;
import de.jtem.moebiusViewer.MoebiusTransform;
import de.jtem.moebiusViewer.util.MoebiusTransformStack;
import de.jtem.moebiusViewer.viewer.AbstractViewer;

public abstract class AbstractTool implements MoebiusTool {

    private static final long serialVersionUID = 1L;


    protected String                label;

    protected MoebiusTransformStack transformPath;

    protected MoebiusShape          context;

    protected AbstractViewer        viewer;
    protected Dimension             viewerSize;

    protected MoebiusTransform    contextTransform        = new MoebiusTransform();
    protected MoebiusTransform    firstContextTransform   = new MoebiusTransform();

    protected Complex             firstPick               = new Complex();
    protected Complex             firstScreenPick         = new Complex();
    protected Complex             newPick                 = new Complex();
    protected Complex             newScreenPick           = new Complex();

    public void draw (MoebiusGraphics g){
    }

    public String getLabel () {
	return label;
    } 

    public void setLabel (String v) {
	this.label = v;
    } 

    public MoebiusShape getContext() {
	return context;
    }

    public void setContext( MoebiusShape context ) {
	this.context = context;
    }

    public AbstractViewer getViewer () {
	return viewer;
    } 

    public void setViewer (AbstractViewer v) {
	this.viewer = v;
    } 

    protected void updateContextTransform() {
    
	contextTransform.assign( 1, 0, 0, 0, 0, 0, 1, 0 );

	for( int i=0; i<transformPath.size(); i++ ) {
	    contextTransform.assignTimes( transformPath.elementAt( i   ) );
	}
    }

    public MoebiusTransform getContextTransform () {
	updateContextTransform();
	return contextTransform;
    } 

    public MoebiusTransformStack getTransformPath() {
	return transformPath;
    }

    public void setTransformPath( MoebiusTransformStack v ) {
	this.transformPath = new MoebiusTransformStack( v );

	updateContextTransform();
    } 
  
    public Object clone() throws CloneNotSupportedException {
	return super.clone();
    }

    // 
    // MouseListener stuff
    // 

    public void mouseClicked (MouseEvent e){}

    public void mouseEntered (MouseEvent e){}

    public void mouseExited (MouseEvent e){}

    public void mousePressed (MouseEvent e) {
	viewerSize=viewer.getSize(viewerSize);
	updateContextTransform();

	firstContextTransform.assign( contextTransform );

	firstScreenPick.assign( (double) e.getX (), (double) (viewerSize.height-e.getY ()) );

	firstContextTransform.applyInverseTo( firstScreenPick, firstPick );

	newPick.assign( firstPick );
	newScreenPick.assign( firstScreenPick );
    } 

    public void mouseReleased (MouseEvent e) {   
    } 

    // 
    // End MouseListener stuff
    // 
    // 
    // MouseMotionListener stuff
    // 

    public void mouseDragged (MouseEvent e){
	viewerSize=viewer.getSize(viewerSize);
        newScreenPick.re = (double) e.getX ();
	newScreenPick.im = (double) (viewerSize.height-e.getY ());
    
	contextTransform.applyInverseTo( newScreenPick, newPick );
    }
  
    public void mouseMoved (MouseEvent e){}
  
}
