/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.tool;

import java.awt.event.MouseEvent;
import java.util.Vector;

import de.jtem.mfc.geometry.ComplexProjective1;
import de.jtem.moebiusViewer.AbstractPicker;
import de.jtem.moebiusViewer.MoebiusGraphics;
import de.jtem.moebiusViewer.MoebiusTransform;

public class ThreePointTransformTool extends AbstractMoebiusTransformerTool {

    private static final long     serialVersionUID = 1L;
 
    boolean                       drawCenter = false;
 
    ComplexProjective1            zero  = new ComplexProjective1( 0, 0, 1, 0 );
    ComplexProjective1            one   = new ComplexProjective1( 1, 0, 1, 0 );
    ComplexProjective1            infty = new ComplexProjective1( 1, 0, 0, 0 );
    ComplexProjective1            currentPoint;
    ComplexProjective1            tZero  = new ComplexProjective1( );
    ComplexProjective1            tOne   = new ComplexProjective1( );
    ComplexProjective1            tInfty = new ComplexProjective1( );

    ComplexProjective1            nZero  = new ComplexProjective1( );
    ComplexProjective1            nOne   = new ComplexProjective1( );
    ComplexProjective1            nInfty = new ComplexProjective1( );

    MoebiusTransform                     u = new MoebiusTransform();
    MoebiusTransform                     v = new MoebiusTransform();
    MoebiusTransform                     w = new MoebiusTransform();

    public ThreePointTransformTool () {
        label = "three point transform";
    }

    public Vector getTools () {
        return null;
    } 

    public void draw (MoebiusGraphics G) {

	super.draw( G );

	java.awt.Color      c = G.getColor ();


	G.setPointRadius( 5 );

        G.setColor( java.awt.Color.red );
	
	G.point( zero );  G.text( zero, "zero" );
	G.point( one );   G.text( one, "one" );
	G.point( infty ); G.text( infty, "infty" );
    } 

    public void mousePressed (MouseEvent e) {

	super.mousePressed( e );

	tZero. assignTimes( contextTransform, zero  );
	tOne.  assignTimes( contextTransform, one   );
	tInfty.assignTimes( contextTransform, infty );

	nZero. assign( tZero  );
	nOne.  assign( tOne   );
	nInfty.assign( tInfty );

	u.assign( tZero, tOne, tInfty);
	v.assignInvert( u );

	AbstractPicker pickerContext = viewer.getPickerContext();

	pickerContext. setTransformStack( transformPath );

	pickerContext.initPicking ((int) e.getX (), (int) e.getY ());

	pickerContext.setPointRadius( 5 );

        currentPoint = null; 

        pickerContext.point( zero );
        if( pickerContext.getIsPicked() ) {
            currentPoint = nZero;
        } 
        else{
            pickerContext.point( one );
	    if( pickerContext.getIsPicked() ) {
	      currentPoint = nOne;
            } 
            else{
	      
	      pickerContext.point( infty );
	      if( pickerContext.getIsPicked() ) {
		currentPoint = nInfty;
               }
            }
            
        }
        //System.out.println("CurrentPoint: " + currentPoint);
   } 

    public void mouseReleased (MouseEvent e) {
        super.mouseReleased (e);
        currentPoint = null;
        viewer.repaint ();
    } 

    public void mouseDragged (MouseEvent e) {
      super.mouseDragged (e);

      if (currentPoint != null){
	
	  currentPoint.assign(newScreenPick.re, newScreenPick.im, 1.0, 0.0);
	  
	  w.assign( nZero, nOne, nInfty);

	  currentTransform.assignDivide( firstTransform, firstContextTransform );
	  currentTransform.assignTimes( w );
	  currentTransform.assignTimes( v );
	  currentTransform.assignTimes(  firstContextTransform );

	  viewer.repaint ();
        } 
    }

}

