/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.util;

import java.io.Serializable;

import de.jtem.moebiusViewer.MoebiusTransform;

public class MoebiusTransformStack 
    implements Serializable,Cloneable {
    private static final long     serialVersionUID = 1L;

    MoebiusTransform [] stack;

    int size = 0;

    public MoebiusTransformStack() {
	stack = new MoebiusTransform[10];
    }

    public MoebiusTransformStack( MoebiusTransformStack transformStack ) {
	size = transformStack.size;

	stack = new MoebiusTransform[ size ];

	System.arraycopy( transformStack.stack, 0, stack, 0, size );
    }

    public int size() {
	return size;
    }

    public void setSize( int newSmallerSize ) {
	if( newSmallerSize > size ) 
	    throw new IllegalArgumentException();
	while( size > newSmallerSize )
	    pop();
    }

    public boolean empty() {
	return size == 0;
    }

    public MoebiusTransform push( MoebiusTransform transform ) {

	if( stack.length == size ) {
	    MoebiusTransform [] newStack = new MoebiusTransform [2*size ];
	    System.arraycopy( stack, 0, newStack, 0, size );
	    stack = newStack;
	}

	stack[size++] = transform;

	return transform;
    }
    
    public MoebiusTransform pop() {

	if( size == 0 )
	    throw new java.util.EmptyStackException();

	MoebiusTransform poppedTransform = stack[size-1];

	stack[--size] = null;

	return poppedTransform;
    }

    public MoebiusTransform peek() {

	if( size == 0 )
	    throw new java.util.EmptyStackException();

	return stack[size-1];
    }


    public MoebiusTransform elementAt( int index ) {
	if( index < 0 || index >= size )
	    throw new java.util.NoSuchElementException();

	return stack[ index ];
    }

    public void clear() {
	java.util.Arrays.fill( stack, null );
	size = 0;	
    }

    public Object clone() {
	return new MoebiusTransformStack( this );
    }
}

