/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.viewer;

import java.awt.Color;
import java.awt.Frame;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import de.jtem.mfc.group.Moebius;
import de.jtem.moebiusViewer.Attributes;
import de.jtem.moebiusViewer.Attributes2D;
import de.jtem.moebiusViewer.moebius.JMoebiusViewer;
import de.jtem.moebiusViewer.shape.Circle;
import de.jtem.moebiusViewer.shape.MoebiusTransformer;
import de.jtem.moebiusViewer.shape.Polygon;
import de.jtem.moebiusViewer.shape.ShapeSet;

public class Example extends Frame {

  private static final long serialVersionUID = 1L;
  
  static JMoebiusViewer aViewer = new JMoebiusViewer ();
	
  static {
  	Attributes a = new Attributes2D();
  	a.setAntiAliasing(true);
  	aViewer.setDoubleBuffered(true);
  	aViewer.setAttributes(a);
  	aViewer.setBackground(new java.awt.Color(225,225,225));
  	
  };
  static ShapeSet shapeSet = new ShapeSet();
  static Moebius t = new Moebius( 2, 0, 0, 0, 0,0, 1, 0 ); // scaling by 2
  static MoebiusTransformer  transformShape = new MoebiusTransformer(  t  );

  public Example (String aString) {
    super (aString);

    //setting up a "render tree"
    transformShape.setShape( shapeSet );
    aViewer.add ( transformShape );
    
    // producing a line set with 2 polygons with 5 points each
    Polygon p = new Polygon( );
    p.assignCoords( new double[] { 1,1,  1,-1, -1,-1, -1,1 } );
    p.setClosed( true );
    p.setPointRadius( 3 );
    p.setColor( Color.red );
    
    //adding the polygon 2 times
    aViewer.add (p); 
    shapeSet.add (p);

    Circle circle = new Circle();

    circle.setRadius( 1.2 );

    aViewer.add ( circle );
    
    add (aViewer);
    pack ();
  }

  public static void main (String[] s) {
    WindowListener      l = new WindowAdapter () {
      public void windowClosing (WindowEvent e) {
        System.exit (0);
      }

    };
    ComponentListener   c = new ComponentAdapter () {
      public void componentResized (ComponentEvent e) {
        aViewer.encompass ();
      }

    };
  
    Example  theExample = new Example ("Moebius Viewer Example");

    theExample.addWindowListener (l);
    theExample.addComponentListener (c);

    theExample.pack();
    theExample.setVisible(true);
  }
}

