/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.moebiusViewer.viewer;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.PrintJob;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.awt.image.ImageObserver;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.io.Serializable;

import de.jtem.moebiusViewer.AbstractBounder;
import de.jtem.moebiusViewer.AbstractDrawer;
import de.jtem.moebiusViewer.AbstractPicker;
import de.jtem.moebiusViewer.Attributes;
import de.jtem.moebiusViewer.MoebiusShape;
import de.jtem.moebiusViewer.shape.Rectangle;

public class ViewerCanvas extends AbstractViewerCanvas
    implements Serializable, ActionListener,
	       PropertyChangeListener, ImageObserver {

    private static final long serialVersionUID = 1L;

    public ViewerCanvas( AbstractDrawer drawer, 
			AbstractPicker picker, 
			AbstractBounder bounder ) {

	super( drawer, picker, bounder );
    }


    protected void processTypedKey( char typedKey ) {

	switch( typedKey ) {

	    case 'E':
		encompass( true );
		repaint();
		break;
	    case 'e':
		encompass();
		repaint();
		break;
	    case 'i':
		invertInUnitCircle();
		repaint ();
		break;
	    case 'c':
		center ();
		repaint ();
		break;
	    case 'w':
		writePS();
		break;
	    case 'W':
		writePS( viewer.getBound() );
		break;
	    case 'P':
		writePPM();
		break;
	    case 'p':

		Component component = this;
		
		while ( component.getParent() != null )
		    component = component.getParent();
		
		try {
		    PrintJob p=getToolkit().getPrintJob((Frame)component, 
							"Print ...", null);
		    paint(p.getGraphics());
		    p.end();
		    
		} catch( ClassCastException cce ) {
		    System.out.println("Sorry viewer must be in a frame!");
		}
		break;
	default:
	    super.processTypedKey( typedKey );
	}
    }

    public void setFont( Font aFont) {
	viewer.setFont( aFont );
    } 

    public boolean contains( MoebiusShape shape ) {
	return viewer.contains( shape );
    }
    
    public void removeAll() {
	viewer.removeAll();
    }

    public void set( MoebiusShape[] someShapes ) {
	viewer.set( someShapes );
    }

    public void set( int index, MoebiusShape aShape ) {
	viewer.set( index, aShape );
    }

    public void add (MoebiusShape aShape) {
	viewer.add (aShape);
    } 

    public void add (MoebiusShape[] aShape) {
	viewer.add (aShape);
    } 

    public void remove (MoebiusShape aShape) {
	viewer.remove (aShape);
    } 

    public void remove (MoebiusShape[] aShape) {
	viewer.remove (aShape);
    } 

    public void writePS( MoebiusShape shape, Rectangle aBound ) {
	viewer.writePS( shape, aBound );
    }
    
    public void writePS( Rectangle aBound ) {
	viewer.writePS( aBound );
    }
    
    public void writePS( Dimension d ) {
	viewer.writePS( d );
    }

    public void writePS() {
	viewer.writePS();
    }

    public void paintOffScreen( BufferedImage anImage ) {
	viewer.paintOffScreen( anImage );
    }
  
    public void writePPM( String filename ) throws IOException {
	viewer.writePPM( filename );
    }

    public void writePPM( File file ) throws IOException {
	viewer.writePPM( file );
    }

    public void writePPM() {
	viewer.writePPM();
    }
     
    public void encompass () {
	viewer.encompass();
    }

    public void encompass( boolean reset ) {
	viewer.encompass( reset );
    }

    public void center () {
	viewer.center();
    }

    public void invertInUnitCircle () {
	viewer.invertInUnitCircle();
    }

    public void setAttributes( Attributes a){
	viewer.setAttributes( a );
    }

    public Attributes getAttributes(){
	return viewer.getAttributes();	
    }
}

