/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2002-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.riemann.analyticContinuation;

import de.jtem.blas.ComplexVector;
import de.jtem.mfc.field.Complex;


public class AnalyticContinuation extends AnalyticIntegrator {

	public AnalyticContinuation( AlgebraicCurve curve ) {
		super( curve, new AnalyticContinuousIntegratorODE( curve ) );
	}
	
	public void startContinuation(Complex lambda, Complex mu ) {
		super.startContinuation( lambda, mu );
	}

	private final Complex P = new Complex();

	public void continueOnMuPlaneTo(Complex Q, Complex lambdaAtQ ) {
		P.assign(I[3], I[4]);

		continueAlongSegment( P, Q, false );
		
		lambdaAtQ.assign(I[1], I[2]);
	}

	public void continueOnLambdaPlaneTo(Complex Q, Complex muAtQ ) {
		P.assign(I[1], I[2]);

		continueAlongSegment( P, Q, true );

		muAtQ.assign(I[3], I[4]);
	}

	
	public final void continueAlongPathOnLambdaPlane(
			ComplexVector path,
		Complex muAtStartPoint,
		Complex muAtEndPoint ) {
		continueAlongPath(path, muAtStartPoint, muAtEndPoint, true );
	}
	

	public final void continueAlongPathOnMuPlane(
			ComplexVector path,
			Complex lambdaAtStartPoint,
			Complex lambdaAtEndPoint ) {
		continueAlongPath( path, lambdaAtStartPoint, lambdaAtEndPoint, false );
	}

}
